#include <sys/param.h>
#include <sys/conf.h>
#include <sys/kernel.h>
#include <sys/malloc.h>
#include <sys/module.h>
#include <sys/systm.h>

#define DEV_MEM "test_mem"

MALLOC_DECLARE(TEST_MEM);
MALLOC_DEFINE(TEST_MEM, "test_mem", "test_mem driver");

/* Test application to allocate contiguous memory then free it */
static int test_application()
{
    int i = 0;
    int array[10] = {2097152, 1024, 2101248, 1024, 2097152, 2101248,
                     2101248, 2097152, 2101248, 1024};
    int array1[10] = {1024, 2101248, 2097152, 2101248, 2101248, 2097152,
                      1024, 2101248, 1024, 2097152};
    void *mem_blocks[10];

    for (i = 0; i < 10; i++)
    {
        mem_blocks[i] = contigmalloc(array[i], TEST_MEM, 0, 0, ~0, PAGE_SIZE, 0);
        if (!mem_blocks[i])
        {
            printf("%s:%d Unable to allocate contiguous memory slab \n", __func__,
                   __LINE__);
            return -1;
        }
    }

    for (i = 0; i < 10; i++)
        contigfree(mem_blocks[i], array[i], TEST_MEM);

    for (i = 0; i < 10; i++)
    {
        mem_blocks[i] = contigmalloc(array1[i], TEST_MEM, 0, 0, ~0, PAGE_SIZE, 0);
        if (!mem_blocks[i])
        {
            printf("%s:%d Unable to allocate contiguous memory slab \n", __func__,
                   __LINE__);
            return -1;
        }
    }

    for (i = 0; i < 10; i++)
        contigfree(mem_blocks[i], array1[i], TEST_MEM);

    return 0;
}

static int mem_modevent(module_t mod __unused,
                            int type,
                            void *data __unused)
{

    switch (type)
    {
        case MOD_LOAD:
            test_application();
            return (0);
        case MOD_UNLOAD:
            return (0);
        default:
            return (EOPNOTSUPP);
    }
}

DEV_MODULE(test_mem, mem_modevent, NULL);
MODULE_VERSION(test_mem, 1);
